@@ -1,7 +1,11 @@
 
 package provide pdwindow 0.1
 
+package require pd_connect
+
 namespace eval ::pdwindow:: {
+    variable maxlogbuffer 21000 ;# if the logbuffer grows beyond this number, cut it
+    variable keeplogbuffer 1000 ;# if the logbuffer gets automatically cut, keep this many elements
     variable logbuffer {}
     variable tclentry {}
     variable tclentry_history {"console show"}
@@ -10,7 +14,9 @@ namespace eval ::pdwindow:: {
     variable logmenuitems
     variable maxloglevel 4
 
-    variable lastlevel 0
+    # private variables
+    variable _lastlevel 0       ;# loglevel of last post (for automatic endpost level)
+    variable _curlogbuffer 0    ;# number of \n currently in the logbuffer
 
     namespace export create_window
     namespace export pdtk_post
@@ -56,10 +62,49 @@ proc ::pdwindow::busyrelease {} {
 
 proc ::pdwindow::buffer_message {object_id level message} {
     variable logbuffer
-    lappend logbuffer $object_id $level $message
+    variable maxlogbuffer
+    variable keeplogbuffer
+    variable _curlogbuffer
+    lappend logbuffer [list $object_id $level $message]
+    set lfi 0
+    while { [set lfi [string first "\n" $message $lfi]] >= 0 } {
+        incr lfi
+        incr _curlogbuffer
+    }
+    # what we are actually counting here is not the number of *lines* in the logbuffer,
+    # but the number of buffer_messages, which is much higher
+    # e.g. printing a 10 element list ([1 2 3 4 5 6 7 8 9 10( -> [print])
+    # will add 22 messages (one prefix, one per atom, one per space-between-atoms, one LF)
+    # LATER we could try to track "\n"
+    # buffer-size limiting is only done if maxlogbuffer is > 0
+    if {$maxlogbuffer > 0 && $_curlogbuffer > $maxlogbuffer} {
+        # so we now have more lines (counting "\n") in the buffer than we actually want
+        set keeplines ${keeplogbuffer}
+        if {$keeplines > $maxlogbuffer} {set keeplines $maxlogbuffer}
+        set count 0
+        set keepitems 0
+        # check how many elements we need to save to keep ${keeplines} lines
+        foreach x [lreverse $logbuffer] {
+            set x [lindex $x 2]
+            set lfi 0
+            while { [set lfi [string first "\n" $x $lfi] ] >= 0} { incr lfi
+                incr count
+            }
+            if { $count >= $keeplines } {
+                break
+            }
+            incr keepitems
+        }
+        set logbuffer [lrange $logbuffer end-$keepitems end]
+        set msg [format [_ "dropped %d lines from the Pd window" ] [expr $_curlogbuffer - $count]]
+        set _curlogbuffer 0
+        ::pdwindow::verbose 10 "$msg\n"
+        ::pdwindow::filter_logbuffer
+    }
 }
 
 proc ::pdwindow::insert_log_line {object_id level message} {
+    set message [subst -nocommands -novariables $message]
     if {$object_id eq ""} {
         .pdwindow.text.internal insert end $message log$level
     } else {
@@ -73,22 +118,29 @@ proc ::pdwindow::insert_log_line {object_id level message} {
     }
 }
 
-# this has 'args' to satisfy trace, but its not used
-proc ::pdwindow::filter_buffer_to_text {args} {
+proc ::pdwindow::filter_logbuffer {} {
     variable logbuffer
     variable maxloglevel
     .pdwindow.text.internal delete 0.0 end
     set i 0
-    foreach {object_id level message} $logbuffer {
-        if { $level <= $::loglevel || $maxloglevel == $::loglevel} {
-            insert_log_line $object_id $level $message
+    foreach logentry $logbuffer {
+        foreach {object_id level message} $logentry {
+            if { $level <= $::loglevel || $maxloglevel == $::loglevel} {
+                insert_log_line $object_id $level $message
+            }
         }
         # this could take a while, so update the GUI every 10000 lines
         if { [expr $i % 10000] == 0} {update idletasks}
         incr i
     }
     .pdwindow.text.internal yview end
-    ::pdwindow::verbose 10 "The Pd window filtered $i lines\n"
+    return $i
+}
+# this has 'args' to satisfy trace, but its not used
+proc ::pdwindow::filter_buffer_to_text {args} {
+    set i [::pdwindow::filter_logbuffer]
+    set msg [format [_ "the Pd window filtered %d lines" ] $i ]
+    ::pdwindow::verbose 10 "$msg\n"
 }
 
 proc ::pdwindow::select_by_id {args} {
@@ -104,7 +156,7 @@ proc ::pdwindow::select_by_id {args} {
 # information about the patches they are building
 proc ::pdwindow::logpost {object_id level message} {
     variable maxloglevel
-    variable lastlevel $level
+    variable _lastlevel $level
 
     buffer_message $object_id $level $message
     if {[llength [info commands .pdwindow.text.internal]] &&
@@ -117,7 +169,7 @@ proc ::pdwindow::logpost {object_id level message} {
         after idle .pdwindow.text.internal yview end
     }
     # -stderr only sets $::stderr if 'pd-gui' is started before 'pd'
-    if {$::stderr} {puts stderr $message}
+    if {$::stderr} {puts -nonewline stderr $message}
 }
 
 # shortcuts for posting to the Pd window
@@ -132,8 +184,8 @@ proc ::pdwindow::pdtk_post {message} {post $message}
 
 proc ::pdwindow::endpost {} {
     variable linecolor
-    variable lastlevel
-    logpost {} $lastlevel "\n"
+    variable _lastlevel
+    logpost {} $_lastlevel "\n"
     set linecolor [expr ! $linecolor]
 }
 
@@ -148,6 +200,7 @@ proc ::pdwindow::verbose {level message} {
 # clear the log and the buffer
 proc ::pdwindow::clear_console {} {
     variable logbuffer {}
+    variable _curlogbuffer 0
     .pdwindow.text.internal delete 0.0 end
 }
 
@@ -157,12 +210,14 @@ proc ::pdwindow::save_logbuffer_to_file {} {
     set filename [tk_getSaveFile -initialfile "pdwindow.txt" -defaultextension .txt]
     if {$filename eq ""} return; # they clicked cancel
     set f [open $filename w]
-    puts $f "Pd $::PD_MAJOR_VERSION.$::PD_MINOR_VERSION.$::PD_BUGFIX_VERSION.$::PD_TEST_VERSION on $::windowingsystem"
-    puts $f "Tcl/Tk [info patchlevel]"
-    puts $f "------------------------------------------------------------------------------"
-    foreach {object_id level message} $logbuffer {
-        puts $f $message
+    puts $f "Pd $::PD_MAJOR_VERSION.$::PD_MINOR_VERSION-$::PD_BUGFIX_VERSION$::PD_TEST_VERSION on $::tcl_platform(os) $::tcl_platform(machine)"
+    puts $f "--------------------------------------------------------------------------------"
+    foreach logentry $logbuffer {
+        foreach {object_id level message} $logentry {
+            puts -nonewline $f $message
+        }
     }
+    ::pdwindow::post "saved console to: $filename\n"
     close $f
 }
 # this has 'args' to satisfy trace, but its not used
@@ -231,7 +286,7 @@ proc ::pdwindow::pdwindow_bindings {} {
     } else {
         # TODO should it possible to close the Pd window and keep Pd open?
         bind .pdwindow <$::modifier-Key-w>   "wm iconify .pdwindow"
-        wm protocol .pdwindow WM_DELETE_WINDOW "pdsend \"pd verifyquit\""
+        wm protocol .pdwindow WM_DELETE_WINDOW "::pd_connect::menu_quit"
     }
 }
 
@@ -337,7 +392,7 @@ proc ::pdwindow::create_window {} {
     } else {
         wm minsize .pdwindow 400 51
     }
-    wm geometry .pdwindow =500x400+20+50
+    wm geometry .pdwindow =500x400
 
     frame .pdwindow.header -borderwidth 1 -relief flat -background lightgray
     pack .pdwindow.header -side top -fill x -ipady 5
@@ -358,15 +413,13 @@ proc ::pdwindow::create_window {} {
     label .pdwindow.header.ioframe.iostate \
         -text [_ "Audio off"] -borderwidth 1 \
         -background lightgray -foreground black \
-        -takefocus 0 \
-        -font {$::font_family -14}
+        -takefocus 0
 
 # DIO error label
     label .pdwindow.header.ioframe.dio \
         -text [_ "Audio I/O error"] -borderwidth 1 \
         -background lightgray -foreground lightgray \
-        -takefocus 0 \
-        -font {$::font_family -14}
+        -takefocus 0
 
     pack .pdwindow.header.ioframe.iostate .pdwindow.header.ioframe.dio \
         -side top
@@ -395,7 +448,7 @@ proc ::pdwindow::create_window {} {
     frame .pdwindow.tcl -borderwidth 0
     pack .pdwindow.tcl -side bottom -fill x
     # TODO this should use the pd_font_$size created in pd-gui.tcl
-    text .pdwindow.text -relief raised -bd 2 -font {$::font_family -12} \
+    text .pdwindow.text -relief raised -bd 2 -font [list $::font_family 10] \
         -highlightthickness 0 -borderwidth 1 -relief flat \
         -yscrollcommand ".pdwindow.scroll set" -width 60 \
         -undo false -autoseparators false -maxundo 1 -takefocus 0
@@ -438,32 +491,12 @@ proc ::pdwindow::create_window_finalize {} {
     # this ought to be called after all elements of the window (including the
     # menubar!) have been created!
     if {![winfo viewable .pdwindow.text]} { tkwait visibility .pdwindow.text }
-}
-
-proc ::pdwindow::configure_window_offset {{winid .pdwindow}} {
-    # on X11 measure the size of the window decoration, so we can open windows at the correct position
-    if {$::windowingsystem eq "x11"} {
-        if {[winfo viewable $winid]} {
-            # wait for possible race-conditions at startup...
-            if {[winfo viewable .pdwindow] && ![winfo viewable .pdwindow.header.pad1]} {
-                tkwait visibility .pdwindow.header.pad1
-            }
-
-            regexp -- {([0-9]+)x([0-9]+)\+(-?[0-9]+)\+(-?[0-9]+)} [wm geometry $winid] -> \
-                _ _ _left _top
-            set ::windowframex [expr {[winfo rootx $winid] - $_left}]
-            set ::windowframey [expr {[winfo rooty $winid] - $_top}]
-
-            #puts "======================="
-            #puts "[wm geometry $winid]"
-            #puts "winfo [winfo rootx $winid] [winfo rooty $winid]"
-            #puts "windowframe: $winid $::windowframex $::windowframey"
-            #puts "======================="
-        }
+    set fontsize [::pd_guiprefs::read menu-fontsize]
+    if {$fontsize != ""} {
+        ::dialog_font::apply .pdwindow $fontsize
     }
 }
 
-
 # this needs to happen *after* the main menu is created, otherwise the default Wish
 # menu is not replaced by the custom Apple menu on OSX
 proc ::pdwindow::configure_menubar {} {
